@@ -29,6 +29,7 @@
 #include <es/base/IInterfaceStore.h>
 #include <es/device/ICursor.h>
 #include <es/naming/IContext.h>
+#include <es/interlocked.h>
 
 #include "eventManager.h"
 
@@ -38,6 +39,11 @@
 
 ICurrentProcess* System();
 
+namespace
+{
+    Interlocked registered = 0;
+};
+
 class EventManager : public IEventQueue
 {
     static const int WIDTH  = 1024; // [check] how to determine this parameter?
@@ -481,6 +487,11 @@ public:
     //
     bool getEvent(InputEvent* event)
     {
+        if (!registered)
+        {
+            return false;
+        }
+
         Synchronized<IMonitor*> method(monitor);
 
         int count = eventRing.read(event, sizeof(InputEvent));
@@ -489,6 +500,11 @@ public:
 
     bool getKeystroke(int* stroke)
     {
+        if (!registered)
+        {
+            return false;
+        }
+
         Synchronized<IMonitor*> method(monitor);
 
         int count = keyRing.read(stroke, sizeof(int));
@@ -497,6 +513,11 @@ public:
 
     bool peekKeystroke(int* stroke)
     {
+        if (!registered)
+        {
+            return false;
+        }
+
         Synchronized<IMonitor*> method(monitor);
 
         int count = keyRing.peek(stroke, sizeof(int));
@@ -505,6 +526,11 @@ public:
 
     unsigned int getButtonState()
     {
+        if (!registered)
+        {
+            return 0;
+        }
+
         Synchronized<IMonitor*> method(monitor);
 
         return (modifiers << 3) |
@@ -515,14 +541,22 @@ public:
 
     void getMousePoint(int& x, int &y)
     {
-        Synchronized<IMonitor*> method(monitor);
+        if (registered)
+        {
+            Synchronized<IMonitor*> method(monitor);
 
-        x = this->x;
-        y = this->y;
+            x = this->x;
+            y = this->y;
+        }
     }
 
     bool keyEvent(const void* data, int size)
     {
+        if (!registered)
+        {
+            return false;
+        }
+
         Synchronized<IMonitor*> method(monitor);
         bool notify(false);
 
@@ -610,6 +644,11 @@ public:
 
     bool mouseEvent(const void* eventData, int size)
     {
+        if (!registered)
+        {
+            return false;
+        }
+
         Synchronized<IMonitor*> method(monitor);
         u8* data = (u8*) eventData;
         bool notify(false);
@@ -667,7 +706,7 @@ public:
     {
         Synchronized<IMonitor*> method(monitor);
 
-        if (eventRing.getUsed() == 0)
+        if (registered && eventRing.getUsed() == 0)
         {
             return monitor->wait(timeout);
         }
@@ -676,7 +715,10 @@ public:
 
     void notify()
     {
-        monitor->notifyAll();
+        if (registered)
+        {
+            monitor->notifyAll();
+        }
     }
 
     bool queryInterface(const Guid& riid, void** objectPtr)
@@ -700,14 +742,24 @@ public:
 
     unsigned int addRef()
     {
-        return ref.addRef();
+        unsigned int count = ref.addRef();
+        if (count == 2)
+        {
+            registered = true;
+        }
+        return count;
     }
 
     unsigned int release()
     {
         unsigned int count = ref.release();
-        if (count == 0)
+        if (count == 1)
         {
+            registered = false;
+        }
+        else if (count == 0)
+        {
+            monitor->notifyAll();
             delete this;
             return 0;
         }
@@ -715,23 +767,30 @@ public:
     }
 };
 
-static void* inputProcess(Handle<IEventQueue> eventQueue)
+static void* inputProcess(Handle<IContext> nameSpace)
 {
     using namespace UsageID;
 
-    Handle<IContext> root = System()->getRoot();
-    Handle<IStream> keyboard(root->lookup("device/keyboard"));
+    // Create Event manager objects.
+    Handle<IEventQueue> eventQueue = new EventManager;
+    ASSERT(eventQueue);
+
+    // register the event queue.
+    Handle<IContext> device = nameSpace->lookup("device");
+    ASSERT(device);
+    IBinding* ret = device->bind("event", static_cast<IEventQueue*>(eventQueue));
+    ASSERT(ret);
+
+    Handle<IStream> keyboard(nameSpace->lookup("device/keyboard"));
     ASSERT(keyboard);
-    Handle<IStream> mouse(root->lookup("device/mouse"));
+    Handle<IStream> mouse(nameSpace->lookup("device/mouse"));
     ASSERT(mouse);
     Handle<ICurrentThread> currentThread = System()->currentThread();
 
     int x;
     int y;
 
-    esReport("start server loop.\n");
-
-    for (;;)
+    while (registered)
     {
         u8 buffer[8];
         long count;
@@ -750,7 +809,6 @@ static void* inputProcess(Handle<IEventQueue> eventQueue)
         ASSERT(currentThread);
         currentThread->sleep(10000000 / 60);
     }
-
     return 0;
 }
 
@@ -771,18 +829,7 @@ int main(int argc, char* argv[])
     Handle<IClassFactory> eventManagerFactory(new(ClassFactory<EventManager>));
     classStore->add(CLSID_EventManager, eventManagerFactory);
 
-    // Create Event manager objects.
-    Handle<IEventQueue> eventQueue = new EventManager;
-    ASSERT(eventQueue);
-    // register the event queue.
-    Handle<IContext> device = nameSpace->lookup("device");
-    ASSERT(device);
-    IBinding* ret = device->bind("event", static_cast<IEventQueue*>(eventQueue));
-    ASSERT(ret);
-
-    TEST(nameSpace->lookup("device/event"));
-
-    inputProcess(eventQueue);
+    inputProcess(nameSpace);
 
     // Unregister Event Manager factory.
     classStore->remove(CLSID_EventManager);
@@ -790,5 +837,6 @@ int main(int argc, char* argv[])
     // Unregister IEventQueue interface.
     interfaceStore->remove(IID_IEventQueue);
 
+    esReport("Event manager is terminated.\n");
     // System()->trace(false);
 }
