@@ -29,6 +29,7 @@ class Formatter
     bool    leftJustified;
     bool    alt;            // show base
     bool    cap;            // uppercase
+    int     mode;
     char    sign;
     int     base;
     char    conversion;     // floating-point conversion type
@@ -53,6 +54,124 @@ class Formatter
 
     static const int MAXBUF = 32;   // 509 is ANSI minimum
 
+    // Floating-Point Output
+    template <typename U, int Bit>
+    int digitlen(int& k, int& dd, U f, U r);
+
+    template <typename U, int Bit>
+    int significantlen(U f, U r);
+
+    static int streamPutc(int c, void* opt)
+    {
+        IStream* stream(static_cast<IStream*>(opt));
+        if (stream)
+        {
+            char ch(static_cast<char>(c));
+            return stream->write(&ch, 1);
+        }
+        return 0;
+    }
+
+    static int stringPutc(int c, void* opt)
+    {
+        std::string* string(reinterpret_cast<std::string*>(opt));
+        if (string)
+        {
+            char ch(static_cast<char>(c));
+            string->append(1, ch);
+            return 1;
+        }
+        return 0;
+    }
+
+public:
+
+    struct Mode
+    {
+        static const int C = 0;
+        static const int ECMAScript = 1;
+    };
+
+    /**
+     * Constructs a new formatter with the specified output function.
+     * The output stream of the formatted output is determined by the specified function.
+     * @param putc the function to print a character.
+     */
+    Formatter(int (*putc)(int, void*), void* opt) throw();
+    /**
+     * Copy constructor.
+     */
+    Formatter(const Formatter& o) throw();
+    /**
+     * Constructs a new formatter with the specified output stream.
+     * The formated strings are written to the specified stream.
+     * @param stream the output stream.
+     */
+    Formatter(IStream* stream) throw();
+    /**
+     * Constructs a new formatter with the specified string.
+     * The formated strings are written to the specified string.
+     * @param string the string.
+     */
+    Formatter(std::string& string) throw();
+    /**
+     * Destructs this object.
+     */
+    ~Formatter();
+
+    /**
+     * Writes a formatted string to the output stream of this object
+     * using the specified format string and arguments.
+     * @param spec the format string.
+     * @param args the arguments referenced by the format specifiers in the format string.
+     */
+    int format(const char* spec, va_list args)  __attribute__ ((format (printf, 2, 0)));
+
+    /**
+     * Writes a formatted string to the output stream of this object
+     * using the specified format string and arguments.
+     * @param spec the format string.
+     * @param ... the arguments referenced by the format specifiers in the format string.
+     */
+    int format(const char* spec, ...) __attribute__ ((format (printf, 2, 3)))
+    {
+        va_list list;
+        int count;
+
+        va_start(list, spec);
+        count = format(spec, list);
+        va_end(list);
+        return count;
+    }
+
+    int print(char c);
+    int print(const char* string);
+    int print(float x);
+    int print(double x);
+
+    int print(short n)
+    {
+        return print((long) n);
+    }
+    int print(unsigned short n)
+    {
+        return print((unsigned long) n);
+    }
+
+    int print(int n)
+    {
+        return print((long) n);
+    }
+    int print(unsigned int n)
+    {
+        return print((unsigned long) n);
+    }
+
+    int print(long n);
+    int print(unsigned long n);
+    int print(long long n);
+    int print(unsigned long long n);
+
     void reset()
     {
         filler = ' ';
@@ -135,6 +254,17 @@ class Formatter
         return !cap;
     }
 
+    int getMode() const
+    {
+        return mode;
+    }
+
+    int setMode(int mode)
+    {
+        this->mode = mode;
+        return this->mode;
+    }
+
     int bin()
     {
         return setBase(2);
@@ -162,7 +292,10 @@ class Formatter
 
     int setBase(int n)
     {
-        base = n;
+        if (2 <= n && n <= 36)
+        {
+            base = n;
+        }
         return base;
     }
 
@@ -177,120 +310,20 @@ class Formatter
         return sign;
     }
 
-    // Floating-Point Output
-    void general();
-    void scientific();
-    void fixed();
-
-    template <typename U, int Bit>
-    int digitlen(int& k, int& dd, U f, U r);
-
-    template <typename U, int Bit>
-    int significantlen(U f, U r);
-
-    static int streamPutc(int c, void* opt)
+    void general()
     {
-        IStream* stream(static_cast<IStream*>(opt));
-        if (stream)
-        {
-            char ch(static_cast<char>(c));
-            return stream->write(&ch, 1);
-        }
-        return 0;
+        conversion = 'g';
     }
 
-    static int stringPutc(int c, void* opt)
+    void scientific()
     {
-        std::string* string(reinterpret_cast<std::string*>(opt));
-        if (string)
-        {
-            char ch(static_cast<char>(c));
-            string->append(1, ch);
-            return 1;
-        }
-        return 0;
+        conversion = 'e';
     }
 
-public:
-    /**
-     * Constructs a new formatter with the specified output function.
-     * The output stream of the formatted output is determined by the specified function.
-     * @param putc the function to print a character.
-     */
-    Formatter(int (*putc)(int, void*), void* opt) throw();
-    /**
-     * Copy constructor.
-     */
-    Formatter(const Formatter& o) throw();
-    /**
-     * Constructs a new formatter with the specified output stream.
-     * The formated strings are written to the specified stream.
-     * @param stream the output stream.
-     */
-    Formatter(IStream* stream) throw();
-    /**
-     * Constructs a new formatter with the specified string.
-     * The formated strings are written to the specified string.
-     * @param string the string.
-     */
-    Formatter(std::string& string) throw();
-    /**
-     * Destructs this object.
-     */
-    ~Formatter();
-
-    /**
-     * Writes a formatted string to the output stream of this object
-     * using the specified format string and arguments.
-     * @param spec the format string.
-     * @param args the arguments referenced by the format specifiers in the format string.
-     */
-    int format(const char* spec, va_list args)  __attribute__ ((format (printf, 2, 0)));
-
-    /**
-     * Writes a formatted string to the output stream of this object
-     * using the specified format string and arguments.
-     * @param spec the format string.
-     * @param ... the arguments referenced by the format specifiers in the format string.
-     */
-    int format(const char* spec, ...) __attribute__ ((format (printf, 2, 3)))
+    void fixed()
     {
-        va_list list;
-        int count;
-
-        va_start(list, spec);
-        count = format(spec, list);
-        va_end(list);
-        return count;
+        conversion = 'f';
     }
-
-    int print(char c);
-    int print(const char* string);
-    int print(float x);
-    int print(double x);
-
-    int print(short n)
-    {
-        return print((long) n);
-    }
-    int print(unsigned short n)
-    {
-        return print((unsigned long) n);
-    }
-
-    int print(int n)
-    {
-        return print((long) n);
-    }
-    int print(unsigned int n)
-    {
-        return print((unsigned long) n);
-    }
-
-    int print(long n);
-    int print(unsigned long n);
-    int print(long long n);
-    int print(unsigned long long n);
 };
 
 #endif  // NINTENDO_ES_FORMATTER_H_INCLUDED
