@@ -514,6 +514,7 @@ Process() :
     tlsSize(0),
     threadCount(0),
     root(0),
+    current(0),
     in(0),
     out(0),
     error(0),
@@ -552,19 +553,10 @@ Process::
          proxy < &syscallTable[INTERFACE_POINTER_MAX];
          ++proxy)
     {
-        if (1 < proxy->addRef())
-        {
-            IInterface* object = static_cast<IInterface*>(proxy->getObject());
-            if (object)
-            {
-#ifdef VERBOSE
-                Reflect::Interface interface = getInterface(proxy->iid);   // XXX Should cache the result.
-                esReport("%s %s %p\n", __func__, interface.getName(), object);
-#endif
-                object->release();
-            }
-        }
-    }
+        proxy->addRef();
+        while (0 < proxy->release())
+            ;
+   }
 
     while (!upcallList.isEmpty())
     {
@@ -915,6 +907,15 @@ exit(int status)
 
     exitValue = status;
 
+    for (SyscallProxy* proxy(syscallTable);
+         proxy < &syscallTable[INTERFACE_POINTER_MAX];
+         ++proxy)
+    {
+        proxy->addRef();
+        while (0 < proxy->release())
+            ;
+    }
+
     // Cancel all the threads. Note do not release threads here as they are
     // stored in syscallTable.
     Thread* thread;
@@ -989,6 +990,35 @@ setRoot(IContext* root)
     }
 }
 
+IContext* Process::
+getCurrent()
+{
+    Monitor::Synchronized method(monitor);
+
+    if (current)
+    {
+        current->addRef();
+    }
+    return current;
+}
+
+void Process::
+setCurrent(IContext* current)
+{
+    Monitor::Synchronized method(monitor);
+
+    IContext* prev(this->current);
+    if (current)
+    {
+        current->addRef();
+    }
+    this->current = current;
+    if (prev)
+    {
+        prev->release();
+    }
+}
+
 IStream* Process::
 getIn()
 {
