@@ -177,16 +177,15 @@ updatePriority()
 }
 
 /** Unlocks all the monitors locked by this thread.
+ * This thread is being locked.
  */
 void Thread::
 unlockAllMonitors()
 {
+    ASSERT(state == TERMINATED);
     while (!monitorList.isEmpty())
     {
-        lock();
         Monitor* monitor = monitorList.getFirst();
-        unlock();
-        monitor->lockCount = 1;
         monitor->unlock();
     }
 }
@@ -235,11 +234,10 @@ exit(void* val)
     ASSERT(state == RUNNING);
     ASSERT(getCurrentThread() == this);
 
-    unlockAllMonitors();
-
     lock();
-    // ClearContext(context);
+    // XXX Clear FPU context
     state = TERMINATED;
+    unlockAllMonitors();
     this->val = val;
     joinPoint.wakeup();
     unlock();
@@ -363,9 +361,15 @@ cancel()
         return;
     }
 
+    if (process)
+    {
+        process->detach(this);
+    }
+
     switch (state)
     {
       case RUNNABLE:
+        unsetRun();
         break;
       case RUNNING:
         sched->runQueueHint = true;    // Hint to scheduler to check run queue
@@ -383,11 +387,11 @@ cancel()
         return;
     }
 
-    // OSClearContext(&context);
+    // XXX Clear FPU context
 
     ASSERT(0 < ref);    // i.e., not detached.
-    unlockAllMonitors();// XXX deadlock
     state = TERMINATED;
+    unlockAllMonitors();
     joinPoint.wakeup();
 
     unlock();
@@ -487,6 +491,9 @@ Thread(void* (*func)(void*), void* param, int priority,
 Thread::
 ~Thread()
 {
+#ifdef VERBOSE
+    esReport("Thread::%s %p\n", __func__, this);
+#endif
     delete[] (u8*) stack;
 }
 
@@ -557,7 +564,14 @@ returnToClient()
     {
         core->tcb->tcb = record->tcb;
         client = record->process;
+        ASSERT(client);
     }
+
+    if (client)
+    {
+        client->load();
+    }
+
     Core::splX(x);
 
     return client;
@@ -574,6 +588,8 @@ leapIntoServer(UpcallRecord* record)
     upcallList.addLast(record);
     core->tcb->tcb = record->tcb;
     server = record->process;
+    ASSERT(server);
+    server->load();
 
     Core::splX(x);
     return server;
